/**
 *  Copyright (c) 2020, Texas Instruments Incorporated
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:\
 *
 *  *   Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  *   Neither the name of Texas Instruments Incorporated nor the names of
 *  its contributors may be used to endorse or promote products derived
 *  from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

export const DEFAULT_CHUNK_SIZE = 4096;
export const MAX_FILE_SIZE = 1024 * 1024;

export interface IStorages {
    SRAM: number[];
    FLASH: number[];
    SFLASH: number[];
    FPGA_CODE: number[];
    OTP: number[];
};

export const storages = new (class implements IStorages {
    SRAM = [0, 0, 0, 0];
    FLASH = [0, 0, 0, 1];
    SFLASH = [0, 0, 0, 2];
    FPGA_CODE = [0, 0, 0, 3];
    OTP = [0, 0, 0, 10];
});

export const file_size_granularities = [256, 1024, 4096, 16384, 65536];

export const FLASH_ERASE_TIME_PER_4K_WORST_CASE = 0.2;

export enum FsFileAccessTypes {
    READ = 0,
    WRITE = 1,
    CREATE = 2,
    WRITE_CREATE_IF_NOT_EXIST = 3
};

export enum FsFileAccessFlags {
    OPEN_FLAG_FAILSAFE = 0x1,
    OPEN_FLAG_SECURE = 0x2,
    OPEN_FLAG_NO_SIGNATURE_TEST = 0x4,
    OPEN_FLAG_STATIC = 0x8,
    OPEN_FLAG_VENDOR = 0x10,
    OPEN_FLAG_PUBLIC_WRITE = 0x20,
    OPEN_FLAG_PUBLIC_READ = 0x40
};

export enum SlFileListProperties {
    OPEN_FILE_COMMIT = 0x1,
    OPEN_BUNDLE_COMMIT = 0x2,
    PENDING_FILE_COMMIT = 0x4,
    PENDING_BUNDLE_COMMIT = 0x8,
    SECURE = 0x10,
    NO_FAILSAFE = 0x20,
    SYSTEM_FILE = 0x40,
    SYSTEM_FILE_HOST_ACCESS = 0x80,
    NO_VALID_COPY = 0x100,
    PUBLIC_WRITE = 0x200,
    PUBLIC_READ = 0x400,
    STATE_OPEN_READ = 0x800,
    STATE_OPEN_WRITE = 0x1000,
    NOSIGNATURE = 0x2000
};

export function SlFileListPropertiesFrom_flags(flags: number) {
    const result: string[] = [];
    for (let i = 0; i < 16; i++) {
        const singleFlag = flags & (1 << i);
        if (singleFlag) {
            result.push(SlFileListProperties[singleFlag]);
        }
    }
    return result;
};

export const file_info_flags_map = [
    SlFileListProperties.STATE_OPEN_WRITE,
    SlFileListProperties.STATE_OPEN_READ,
    SlFileListProperties.NO_FAILSAFE,
    SlFileListProperties.NO_VALID_COPY,
    SlFileListProperties.SYSTEM_FILE,
    SlFileListProperties.OPEN_FILE_COMMIT,
    SlFileListProperties.OPEN_BUNDLE_COMMIT,
    SlFileListProperties.PENDING_FILE_COMMIT,
    SlFileListProperties.PENDING_BUNDLE_COMMIT,
    0,
    SlFileListProperties.SECURE,
    SlFileListProperties.NOSIGNATURE,
    SlFileListProperties.PUBLIC_WRITE,
    SlFileListProperties.PUBLIC_READ,
    0,
    0
];

export enum FsListFlags {
    FS_GET_FILE_ATTRIBUTES = 0x1
};

export const SL_BOOTLDR_OPCODE_ACK = [0, 0xCC];
export const SL_BOOTLDR_OPCODE_NAK = [0, 0x33];
export const SL_BOOTLDR_OPCODE_PING = [0x20];
export const SL_BOOTLDR_OPCODE_START_DOWNLOAD = [0x21];
export const SL_BOOTLDR_OPCODE_CLOSE_FILE = [0x22];
export const SL_BOOTLDR_OPCODE_GET_LAST_STATUS = [0x23];
export const SL_BOOTLDR_OPCODE_SEND_DATA = [0x24];
export const SL_BOOTLDR_OPCODE_DISCONNECT = [0x26];
export const SL_BOOTLDR_OPCODE_GET_STORAGE_LIST = [0x27];
export const SL_BOOTLDR_OPCODE_ERASE = [0x28];
export const SL_BOOTLDR_OPCODE_SET_MIN_FILE_SIZE = [0x29];
export const SL_BOOTLDR_OPCODE_GET_FILE_INFO = [0x2A];
export const SL_BOOTLDR_OPCODE_GET_CHUNK = [0x2B];
export const SL_BOOTLDR_OPCODE_RAW_STORAGE_READ = [0x2C];
export const SL_BOOTLDR_OPCODE_RAW_STORAGE_WRITE = [0x2D];
export const SL_BOOTLDR_OPCODE_FILE_ERASE = [0x2E];
export const SL_BOOTLDR_OPCODE_GET_VERSION_INFO = [0x2F];
export const SL_BOOTLDR_OPCODE_RAW_STORAGE_ERASE = [0x30];
export const SL_BOOTLDR_OPCODE_GET_STORAGE_INFO = [0x31];
export const SL_BOOTLDR_OPCODE_EXEC_FROM_RAM = [0x32];
export const SL_BOOTLDR_OPCODE_SWITCH_2_APPS = [0x33];
export const SL_BOOTLDR_OPCODE_PROGRAMMING = [0x34];
export const SL_BOOTLDR_OPCODE_GET_FILE_LIST = [0x35];
export const SL_BOOTLDR_OPCODE_RENAME_FILE = [0x36];
export const SL_BOOTLDR_OPCODE_GET_FS_USAGE = [0x37];
export const SL_BOOTLDR_OPCODE_FACTORY_DEFAULTS = [0x38];
export const SL_BOOTLDR_OPCODE_SET_DEVICE_PARAM = [0x39];
export const SL_BOOTLDR_OPCODE_GET_DEVICE_PARAM = [0x3A];

export const SL_BOOTLDR_OPCODE_RET_SUCCESS = 0x40;
export const SL_BOOTLDR_OPCODE_RET_ACCESS_IN_PROGRESS = 0x4B;

export const COMMAND_RET_FAIL_OTP_MAC_ADDRESS_VALIDATION_ERROR = 0x52;
export const COMMAND_RET_FAIL_OTP_CONFIRM_SIGNATURE_ERROR = 0x53;
export const COMMAND_RET_FAIL_OTP_ALREADY_LOCKED_ERROR = 0x54;
export const COMMAND_RET_FAIL_OTP_GENERAL_WRITE_ERROR = 0x55;

export function FS_ERROR(n: number) {
    return -10240 - n;
};

export enum fs_error_codes {
    FS_ERROR_EXTRACTION_WILL_START_AFTER_RESET = FS_ERROR(1),
    FS_ERROR_NO_CERTIFICATE_STORE = FS_ERROR(2),
    FS_FILE_IMAGE_SHOULD_BE_AUTHENTICATE = FS_ERROR(3),
    FS_FILE_IMAGE_SHOULD_BE_ENCRYPTED = FS_ERROR(4),
    FS_FILE_IMAGE_CANT_BE_ENCRYPTED = FS_ERROR(5),
    FS_DEVELOPMENT_BOARD_WRONG_MAC = FS_ERROR(6),
    FS_FILE_DEVICE_NOT_SECURED = FS_ERROR(7),
    FS_ERR_INVALID_SYSTEM_FILE_ACCESS = FS_ERROR(8),
    FS_IMAGE_EXTRACT_EXPECTING_USER_KEY = FS_ERROR(9),
    FS_IMAGE_EXTRACT_FAILED_TO_CLOSE_FILE = FS_ERROR(10),
    FS_IMAGE_EXTRACT_FAILED_TO_WRITE_FILE = FS_ERROR(11),
    FS_IMAGE_EXTRACT_FAILED_TO_OPEN_FILE = FS_ERROR(12),
    FS_IMAGE_EXTRACT_FAILED_TO_GET_IMAGE_HEADER = FS_ERROR(13),
    FS_IMAGE_EXTRACT_FAILED_TO_GET_IMAGE_INFO = FS_ERROR(14),
    FS_IMAGE_EXTRACT_SET_ID_NOT_EXIST = FS_ERROR(15),
    FS_IMAGE_EXTRACT_FAILED_TO_DELETE_FILE = FS_ERROR(16),
    FS_IMAGE_EXTRACT_FAILED_TO_FORMAT_FS = FS_ERROR(17),
    FS_IMAGE_EXTRACT_FAILED_TO_LOAD_FS = FS_ERROR(18),
    FS_IMAGE_EXTRACT_FAILED_TO_GET_DEV_INFO = FS_ERROR(19),
    FS_IMAGE_EXTRACT_FAILED_TO_DELETE_STORAGE = FS_ERROR(20),
    FS_IMAGE_EXTRACT_INCORRECT_IMAGE_LOCATION = FS_ERROR(21),
    FS_IMAGE_EXTRACT_FAILED_TO_CREATE_IMAGE_FILE = FS_ERROR(22),
    FS_IMAGE_EXTRACT_FAILED_TO_INIT = FS_ERROR(23),
    FS_IMAGE_EXTRACT_FAILED_TO_LOAD_FILE_TABLE = FS_ERROR(24),
    FS_IMAGE_EXTRACT_LOGGER_ERROR = FS_ERROR(25),
    FS_IMAGE_EXTRACT_ILLEGAL_COMMAND = FS_ERROR(26),
    FS_IMAGE_EXTRACT_FAILED_TO_WRITE_FAT = FS_ERROR(27),
    FS_IMAGE_EXTRACT_FAILED_TO_RET_FACTORY_DEFAULT = FS_ERROR(28),
    FS_IMAGE_EXTRACT_FAILED_TO_READ_NV = FS_ERROR(29),
    FS_PROGRAMMING_IMAGE_NOT_EXISTS = FS_ERROR(30),
    FS_PROGRAMMING_IN_PROCESS = FS_ERROR(31),
    FS_PROGRAMMING_ALREADY_STARTED = FS_ERROR(32),
    FS_SECURITY_ALERT_CERTIFICATE_IN_THE_CHAIN_IS_REVOKED = FS_ERROR(33),
    FS_ERROR_INIT_CERTIFICATE_STORE = FS_ERROR(34),
    FS_PROGRAMMING_ILLEGAL_FILE = FS_ERROR(35),
    FS_PROGRAMMING_NOT_STARTED = FS_ERROR(36),
    FS_IMAGE_EXTRACT_NO_FILE_SYSTEN = FS_ERROR(37),
    FS_ERROR_WRONG_INPUT_SIZE = FS_ERROR(38),
    FS_BUNDLE_FILE_SHOULD_BE_CREATED_WITH_FAILSAFE = FS_ERROR(39),
    FS_BUNDLE_NOT_CONTAIN_FILES = FS_ERROR(40),
    FS_BUNDLE_ALREADY_IN_STATE = FS_ERROR(41),
    FS_BUNDLE_NOT_IN_CORRECT_STATE = FS_ERROR(42),
    FS_SOME_BUNDLE_FILES_ARE_OPENED = FS_ERROR(43),
    FS_INCORRECT_FILE_STATE_FOR_OPERATION = FS_ERROR(44),
    FS_ERR_EMPTY_SFLASH = FS_ERROR(45),
    FS_ERR_FILE_IS_NOT_SECURE_AND_SIGN = FS_ERROR(46),
    FS_ERR_ROOT_CA_IS_UNKOWN = FS_ERROR(47),
    FS_FILE_HAS_NOT_BEEN_CLOSE_CORRECTLY = FS_ERROR(48),
    FS_WRONG_SIGNATURE = FS_ERROR(49),
    FS_WRONG_SIGNATURE_OR_CERTIFIC_NAME_LENGTH = FS_ERROR(50),
    FS_NOT_16_ALIGNED = FS_ERROR(51),
    FS_SECURITY_ALERT_CERT_CHAIN_ERROR = FS_ERROR(52),
    FS_FILE_NAME_EXIST = FS_ERROR(53),
    FS_EXTENDED_BUF_ALREADY_ALLOC = FS_ERROR(54),
    FS_FILE_SYSTEM_NOT_SECURED = FS_ERROR(55),
    FS_ERR_OFFSET_NOT_16_BYTE_ALIGN = FS_ERROR(56),
    FS_ERR_FAILED_READ_NVMEM_HEADER = FS_ERROR(57),
    FS_WRONG_FILE_NAME = FS_ERROR(58),
    FS_FILE_SYSTEM_IS_LOCKED = FS_ERROR(59),
    FS_SECURITY_ALLERT = FS_ERROR(60),
    FS_FILE_UNVALID_FILE_SIZE = FS_ERROR(61),
    FS_ERR_TOKEN_IS_NOT_VALID = FS_ERROR(62),
    FS_NO_DEVICE_IS_LOADED = FS_ERROR(63),
    FS_SECURE_CONTENT_SHA_CHECK_FAILED = FS_ERROR(64),
    FS_SECURE_CONTENT_RETRIVE_ASYMETRIC_KEY_ERROR = FS_ERROR(65),
    FS_ERR_OVERLAP_DETECTION_THRESHHOLD = FS_ERROR(66),
    FS_FILE_HAS_RESERVED_NV_INDEX = FS_ERROR(67),
    FS_ERR_MAX_FS_FILES_IS_LARGER = FS_ERROR(68),
    FS_ERR_MAX_FS_FILES_IS_SMALLER = FS_ERROR(69),
    FS_FILE_MAX_SIZE_EXCEEDED = FS_ERROR(70),
    FS_UNVALID_READ_BUFFER = FS_ERROR(71),
    FS_UNVALID_WRITE_BUFFER = FS_ERROR(72),
    FS_ERR_FILE_IMAGE_IS_CORRUPTED = FS_ERROR(73),
    FS_ERR_SIZE_OF_FILE_EXT_EXCEEDED = FS_ERROR(74),
    FS_WARNING_FILE_NAME_NOT_KEPT = FS_ERROR(75),
    FS_ERR_MAX_OPENED_SECURE_FILE_EXCEEDED = FS_ERROR(76),
    FS_ERR_FAILED_WRITE_NVMEM_HEADER = FS_ERROR(77),
    FS_ERR_NO_AVAILABLE_NV_INDEX = FS_ERROR(78),
    FS_ERR_FAILED_TO_ALLOCATE_MEM = FS_ERROR(79),
    FS_OPERATION_BLOCKED_BY_VENDOR = FS_ERROR(80),
    FS_ERR_FAILED_TO_READ_NVMEM_FILE_SYSTEM = FS_ERROR(81),
    FS_ERR_NOT_ENOUGH_STORAGE_SPACE = FS_ERROR(82),
    FS_ERR_INIT_WAS_NOT_CALLED = FS_ERROR(83),
    FS_ERR_FILE_SYSTEM_IS_BUSY = FS_ERROR(84),
    FS_ERR_INVALID_ACCESS_TYPE = FS_ERROR(85),
    FS_ERR_FILE_ALREADY_EXISTS = FS_ERROR(86),
    FS_ERR_PROGRAM_ERR = FS_ERROR(87),
    FS_ERR_NO_ENTRIES_AVAILABLE = FS_ERROR(88),
    FS_ERR_FILE_ACCESS_IS_DIFFERENT = FS_ERROR(89),
    FS_ERR_UNVALID_FILE_MODE = FS_ERROR(90),
    FS_ERR_FAILED_READ_NVFILE = FS_ERROR(91),
    FS_ERR_FAILED_INIT_STORAGE = FS_ERROR(92),
    FS_ERR_FILE_HAS_NO_FAILSAFE = FS_ERROR(93),
    FS_NO_VALID_COPY_EXISTS = FS_ERROR(94),
    FS_ERROR_INVALID_HANDLE = FS_ERROR(95),
    FS_ERROR_FAILED_TO_WRITE = FS_ERROR(96),
    FS_ERROR_OFFSET_OUT_OF_RANGE = FS_ERROR(97),
    FS_ERR_NO_MEMORY = FS_ERROR(98),
    FS_ERR_UNVALID_LENGTH_FOR_READ = FS_ERROR(99),
    FS_ERR_WRONG_FILE_OPEN_FLAGS = FS_ERROR(100),
    FS_ERR_FILE_NOT_EXISTS = FS_ERROR(101),
    FS_IGNORE_COMMIT_ROLLBAC_FLAG = FS_ERROR(102),
    FS_ERR_INVALID_ARGS = FS_ERROR(103),
    FS_ERR_FILE_IS_PENDING_COMMIT = FS_ERROR(104),
    FS_SECURE_CONTENT_SESSION_ALREADY_EXIST = FS_ERROR(105),
    FS_ERR_UNKNOWN = FS_ERROR(106),
    FS_ERR_FILE_NAME_RESERVED = FS_ERROR(107),
    FS_ERR_NO_FILE_SYSTEM = FS_ERROR(108),
    FS_ERR_INVALID_MAGIC_NUM = FS_ERROR(109),
    FS_ERR_FAILED_TO_READ_NVMEM = FS_ERROR(110),
    FS_ERR_NOT_SUPPORTED = FS_ERROR(111),
    FS_ERR_JTAG_IS_OPENED_NO_FORMAT_TO_PRDUCTION = FS_ERROR(112),
    FS_CONFIG_FILE_RET_READ_FAILED = FS_ERROR(113),
    FS_CONFIG_FILE_CHECSUM_ERROR_SECURITY_ALERT = FS_ERROR(114),
    FS_CONFIG_FILE_NO_SUCH_FILE = FS_ERROR(115),
    FS_CONFIG_FILE_MEMORY_ALLOCATION_FAILED = FS_ERROR(116),
    FS_IMAGE_HEADER_READ_FAILED = FS_ERROR(117),
    FS_ERR_CERT_STORE_DOWNGRADE = FS_ERROR(118),
    FS_ERR_PROGRAMMING_IMAGE_NOT_VALID = FS_ERROR(119),
    FS_ERR_PROGRAMMING_IMAGE_NOT_VERIFIED = FS_ERROR(120),
    FS_ERR_RESERVE_SIZE_IS_SMALLER = FS_ERROR(121),
    FS_ERR_WRONG_ALLOCATION_TABLE = FS_ERROR(122),
    FS_ILLEGAL_SIGNATURE = FS_ERROR(123),
    FS_FILE_ALREADY_OPENED_IN_PENDING_STATE = FS_ERROR(124),
    FS_TOKEN_IS_NOT_VALID_SECURITY_ALERT = FS_ERROR(125),
    FS_NOT_SECURE = FS_ERROR(126),
    FS_ERR_RESET_DURING_PROGRAMMING = FS_ERROR(127),
    FS_CONFIG_FILE_RET_WRITE_FAILED = FS_ERROR(128),
    FS_ERR_FILE_IS_ALREADY_OPENED = FS_ERROR(129),
    FS_ERR_FILE_IS_OPEN_FOR_WRITE = FS_ERROR(130),
    FS_ERR_ALERT_CANT_BE_SET_ON_NON_SECURE_DEVICE = FS_ERROR(131),
    FS_WRONG_CERTIFICATE_FILE_NAME = FS_ERROR(132),

    FS_OK = 0,

    FS_COMMAND_RET_FAIL_OTP_MAC_ADDRESS_VALIDATION_ERROR = 82,
    FS_COMMAND_RET_FAIL_OTP_CONFIRM_SIGNATURE_ERROR = 83,
    FS_COMMAND_RET_FAIL_OTP_GENERAL_WRITE_ERROR = 85,
};
